PROGRAM_NAME='TASCAM, DA88, RS-422, BASIC, 12-94, BA'
(*   DATE:05/20/99    TIME:16:33:25    *)
(***********************************************************)
(* The following code block is provided as a guide to      *)
(* programming the device(s) listed above. This is a       *)
(* sample only, and will most likely require modification  *)
(* to be integrated into a master program.                 *)
(*                                                         *)
(* Device-specific protocols should be obtained directly   *)
(* from the equipment manufacturer to ensure compliance    *)
(* with the most current versions. Within the limits       *)
(* imposed by any non-disclosure agreements, AMX will      *)
(* provide protocols upon request.                         *)
(*                                                         *)
(* If further programming assistance is required, please   *)
(* contact your AMX customer support team.                 *)
(*                                                         *)
(***********************************************************)
(* GENERAL FUNCTION LIST -                                 *)
(*   STATUS, TRANSPORTS (PL,ST,FF,REW,SEARCHES,REC)        *)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

DAT    = 1                             (* AXC-232: TASCAM DA-88 *)
PANEL  = 128

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

#IF_NOT_DEFINED XPORT_CONST
#DEFINE XPORT_CONST
PLAY    = 1
STOP    = 2
PAUSE   = 3
FFWD    = 4
REW     = 5
SFWD    = 6
SREV    = 7
REC     = 8
#END_IF

#IF_NOT_DEFINED FEEDBACK_CONST
#DEFINE FEEDBACK_CONST
PLAY_FB    = 241
STOP_FB    = 242
PAUSE_FB   = 243
FFWD_FB    = 244
REW_FB     = 245
SFWD_FB    = 246
SREV_FB    = 247
REC_FB     = 248
#END_IF

#IF_NOT_DEFINED SYSCALL_MISC
#DEFINE SYSCALL_MISC
NO_BUTTON   = 0
NO_FUNCTION = 256
#END_IF

(* vcr will go into stop after rewinding for a certain time *)
DAT_REW_TO_STOP = 1800 (* 3 min *)

(* vcr will go into stop after search rewinding for a certain time *)
DAT_SREV_TO_STOP = 12000 (* 20 min *)

(* button feedback flag *)
DAT_DEFEAT_FEEDBACK = 0

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

DAT_BUFFER[20]

REQ_STATUS_MODE
TAPE_OUT

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)

DEFINE_CALL 'SEND TO DA-88' (DEV,CMD[10])
LOCAL_VAR CHKSUM LAST INDEX
{
  CHKSUM = 0
  INDEX = 1
  LAST = LENGTH_STRING(CMD)
  WHILE (INDEX <= LAST)
  {
    CHKSUM = CHKSUM + CMD[INDEX]
    INDEX = INDEX + 1
  }
  CHKSUM = CHKSUM BAND $FF
  SEND_STRING DEV,"CMD,CHKSUM"
  SEND_STRING 0,"CMD,CHKSUM"
}

(* "TASCAM DAT" CALL -                                       *)
(* controls a DAT deck using the umatic SP series protocol   *)
(* no pause function available (when deck sent to VAR FWD at *)
(*   0 speed, it goes to stop after a few seconds)           *)
(* uses VAR FWD/REV for f-fwd/rewind                         *)
(* uses F-FWD/REW for searches                               *)
(* to stop : send "STOP" then "STANDBY OFF" strings.         *)
(* use 0 as a button channel if the button doesn't exist     *)
(* use 256 as a button channel if the function should not be *)
(*   available to the user                                   *)
DEFINE_CALL 'TASCAM DAT' (DEV,PNL,PLAYB,STOPB,PAUSEB,FFWDB,
                          REWB,SFWDB,SREVB,RECB,FIRST)
LOCAL_VAR OFFSET_FB
{
  (* get offsets if any *)
  IF (FIRST BAND $FF00)
    OFFSET_FB=((FIRST BAND $FF00)/$FF)-PLAY_FB
  ELSE
    OFFSET_FB=0

  PUSH[PNL,PLAYB]
    IF (![DEV,OFFSET_FB+REC_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DEV,"$20,$01")
      SYSTEM_CALL 'FEEDBACK' (DEV,PLAY,FIRST)
    }

  PUSH[PNL,STOPB]
  {
    CANCEL_WAIT 'DAT REW TO STOP'
    CANCEL_WAIT 'DAT SREV TO STOP'
    CALL 'SEND TO DA-88' (DEV,"$20,$00")
    WAIT 2
      CALL 'SEND TO DA-88' (DEV,"$20,$04")
    SYSTEM_CALL 'FEEDBACK' (DEV,STOP,FIRST)
  }

  PUSH[PNL,FFWDB]
    IF ([DEV,OFFSET_FB+STOP_FB] OR [DEV,OFFSET_FB+FFWD_FB] OR
        [DEV,OFFSET_FB+REW_FB] OR (SFWDB AND ([DEV,OFFSET_FB+PLAY_FB]
        OR [DEV,OFFSET_FB+SREV_FB] OR [DEV,OFFSET_FB+SFWD_FB])))
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DEV,"$20,$10")
      SYSTEM_CALL 'FEEDBACK' (DEV,FFWD,FIRST)
    }
    ELSE IF (SFWDB=NO_BUTTON AND
            ([DEV,OFFSET_FB+SREV_FB] OR [DEV,OFFSET_FB+SFWD_FB]))
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DEV,"$21,$12,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SFWD,FIRST)
    }
    ELSE IF (SFWDB=NO_BUTTON AND [DEV,OFFSET_FB+PLAY_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DEV,"$20,$00")
      WAIT 1
        CALL 'SEND TO DA-88' (DEV,"$21,$12,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SFWD,FIRST)
    }

  PUSH[PNL,SFWDB]
    IF ([DEV,OFFSET_FB+STOP_FB] OR
       [DEV,OFFSET_FB+REW_FB] OR [DEV,OFFSET_FB+FFWD_FB] OR
       [DEV,OFFSET_FB+SREV_FB] OR [DEV,OFFSET_FB+SFWD_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DEV,"$21,$12,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SFWD,FIRST)
    }
    ELSE IF ([DEV,OFFSET_FB+PLAY_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DEV,"$20,$00")
      WAIT 1
        CALL 'SEND TO DA-88' (DEV,"$21,$12,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SFWD,FIRST)
    }

  PUSH[PNL,REWB]
    IF ([DEV,OFFSET_FB+STOP_FB] OR [DEV,OFFSET_FB+FFWD_FB] OR
        [DEV,OFFSET_FB+REW_FB] OR (SREVB AND ([DEV,OFFSET_FB+PLAY_FB]
        OR [DEV,OFFSET_FB+SREV_FB] OR [DEV,OFFSET_FB+SFWD_FB])))

    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      IF (DAT_REW_TO_STOP)
        WAIT DAT_REW_TO_STOP 'DAT REW TO STOP'
        {
          CALL 'SEND TO DA-88' (DEV,"$20,$00")
          WAIT 2
            CALL 'SEND TO DA-88' (DEV,"$20,$04")
          SYSTEM_CALL 'FEEDBACK' (DEV,STOP,FIRST)
        }
      CALL 'SEND TO DA-88' (DEV,"$20,$20")
      SYSTEM_CALL 'FEEDBACK' (DEV,REW,FIRST)
    }
    ELSE IF (SREVB=NO_BUTTON AND
            ([DEV,OFFSET_FB+SREV_FB] OR [DEV,OFFSET_FB+SFWD_FB]))
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      IF (DAT_SREV_TO_STOP)
        WAIT DAT_SREV_TO_STOP 'DAT SREV TO STOP'
        {
          CALL 'SEND TO DA-88' (DEV,"$20,$00")
          WAIT 2
            CALL 'SEND TO DA-88' (DEV,"$20,$04")
          SYSTEM_CALL 'FEEDBACK' (DEV,STOP,FIRST)
        }
      CALL 'SEND TO DA-88' (DEV,"$21,$22,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SFWD,FIRST)
    }
    ELSE IF (SREVB=NO_BUTTON AND [DEV,OFFSET_FB+PLAY_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      IF (DAT_SREV_TO_STOP)
        WAIT DAT_SREV_TO_STOP 'DAT SREV TO STOP'
        {
          CALL 'SEND TO DA-88' (DEV,"$20,$00")
          WAIT 2
            CALL 'SEND TO DA-88' (DEV,"$20,$04")
          SYSTEM_CALL 'FEEDBACK' (DEV,STOP,FIRST)
        }
      CALL 'SEND TO DA-88' (DEV,"$20,$00")
      WAIT 1
        CALL 'SEND TO DA-88' (DEV,"$21,$22,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SFWD,FIRST)
    }

  PUSH[PNL,SREVB]
    IF ([DEV,OFFSET_FB+PLAY_FB] OR
       [DEV,OFFSET_FB+REW_FB] OR [DEV,OFFSET_FB+FFWD_FB] OR
       [DEV,OFFSET_FB+SREV_FB] OR [DEV,OFFSET_FB+SFWD_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      IF (DAT_SREV_TO_STOP)
        WAIT DAT_SREV_TO_STOP 'DAT SREV TO STOP'
        {
          CALL 'SEND TO DA-88' (DEV,"$20,$00")
          WAIT 2
            CALL 'SEND TO DA-88' (DEV,"$20,$04")
          SYSTEM_CALL 'FEEDBACK' (DEV,STOP,FIRST)
        }
      CALL 'SEND TO DA-88' (DEV,"$21,$22,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SREV,FIRST)
    }
    ELSE IF ([DEV,OFFSET_FB+PLAY_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      IF (DAT_SREV_TO_STOP)
        WAIT DAT_SREV_TO_STOP 'DAT SREV TO STOP'
        {
          CALL 'SEND TO DA-88' (DEV,"$20,$00")
          WAIT 2
            CALL 'SEND TO DA-88' (DEV,"$20,$04")
          SYSTEM_CALL 'FEEDBACK' (DEV,STOP,FIRST)
        }
      CALL 'SEND TO DA-88' (DEV,"$20,$00")
      WAIT 1
        CALL 'SEND TO DA-88' (DEV,"$21,$22,$74")
      SYSTEM_CALL 'FEEDBACK' (DEV,SREV,FIRST)
    }

  PUSH[PNL,RECB]
    IF ([DEV,OFFSET_FB+STOP_FB] OR [DEV,OFFSET_FB+REC_FB])
    {
      CANCEL_WAIT 'DAT REW TO STOP'
      CANCEL_WAIT 'DAT SREV TO STOP'
      CALL 'SEND TO DA-88' (DAT,"$20,$02")
      SYSTEM_CALL 'FEEDBACK' (DEV,REC,FIRST)
    }

  IF (!DAT_DEFEAT_FEEDBACK)
  {
    [PNL,PLAYB]  = [DEV,OFFSET_FB+PLAY_FB]
    [PNL,STOPB]  = [DEV,OFFSET_FB+STOP_FB]
    [PNL,PAUSEB] = 0
    [PNL,FFWDB]  = [DEV,OFFSET_FB+FFWD_FB] OR
                   (SFWDB=NO_BUTTON AND [DEV,OFFSET_FB+SFWD_FB])
    [PNL,REWB]   = [DEV,OFFSET_FB+REW_FB] OR
                   (SREVB=NO_BUTTON AND [DEV,OFFSET_FB+SREV_FB])
    [PNL,SFWDB]  = [DEV,OFFSET_FB+SFWD_FB]
    [PNL,SREVB]  = [DEV,OFFSET_FB+SREV_FB]
    [PNL,RECB]   = [DEV,OFFSET_FB+REC_FB]
  }
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

CREATE_BUFFER DAT,DAT_BUFFER

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(* STANDARD SYSTEM CALL FORMAT *)
CALL 'TASCAM DAT' (DAT,PANEL,1,2,0,4,5,6,7,8,0)

(* DISCRETE TRANSPORT FUNCTIONS *)
PUSH[PANEL,11]                         (* PLAY *)
  CALL 'SEND TO DA-88' (DAT,"$20,$01")

PUSH[PANEL,12]                         (* STOP *)
{
  CALL 'SEND TO DA-88' (DAT,"$20,$00")
  WAIT 2
    CALL 'SEND TO DA-88' (DAT,"$20,$04")
}

PUSH[PANEL,13]                         (* PAUSE *)
{
  (* NO PAUSE FUNCTION AVAILABLE *)
}

PUSH[PANEL,14]                         (* F-FWD *)
  CALL 'SEND TO DA-88' (DAT,"$21,$12,$74")

PUSH[PANEL,15]                         (* SHUTTLE FWD *)
  CALL 'SEND TO DA-88' (DAT,"$20,$10")

PUSH[PANEL,16]                         (* REW *)
  CALL 'SEND TO DA-88' (DAT,"$21,$22,$74")

PUSH[PANEL,17]                         (* SHUTTLE REV *)
  CALL 'SEND TO DA-88' (DAT,"$20,$20")

PUSH[PANEL,18]                         (* RECORD *)
  CALL 'SEND TO DA-88' (DAT,"$20,$02")


(* OTHER FUNCTIONS *)
PUSH[PANEL,21]                         (* EJECT *)
  CALL 'SEND TO DA-88' (DAT,"$20,$0F")

PUSH[PANEL,22]                         (* LOCAL DISABLE *)
  CALL 'SEND TO DA-88' (DAT,"$00,$0C")

PUSH[PANEL,23]                         (* LOCAL ENABLE *)
  CALL 'SEND TO DA-88' (DAT,"$00,$1D")


(* REQUEST STATUS *)
(* STATUS DATA ARE STORED AS BITS IN 16 BYTES (0-F). IN ORDER TO     *)
(* REQUEST STATUS, YOU MUST KNOW WHICH SPECIFIC STATUS BYTES YOU     *)
(* NEED, AND REQUEST THOSE BYTES (SEE SONY SP DOCUMENTATION FOR      *)
(* SPECIFIC STATUS BIT INFORMATION). ALSO, THE "STATUS DATA" COMMAND *)
(* SPECIFIES THE NUMBER OF STATUS DATA BYTES IT IS SENDING, BUT NOT  *)
(* THE IDENTITY OF THOSE BYTES (0-F); THUS, WHEN RECEIVING STATUS    *)
(* DATA YOU MUST TRACK WHICH STATUS DATA BYTES THAT YOU REQUESTED.   *)

(* THE COMMAND TO REQUEST STATUS IS "$61,$20", FOLLOWED BY 1 DATA    *)
(* BYTE: THE HIGH 4 BITS INDICATE THE IDENTITY OF THE FIRST BYTE     *)
(* BEING REQUESTED, AND THE LOW 4 BITS INDICATE THE TOTAL NUMBER OF  *)
(* BYTES, INCLUDING THE FIRST BYTE, THAT ARE BEING REQUESTED.        *)

(* EXAMPLES- *)
PUSH[PANEL,31]                         (* TRANSPORT STATUS (BYTES 1-2) *)
{
  REQ_STATUS_MODE = 1
  CALL 'SEND TO DA-88' (DAT,"$61,$20,$12")
}
(* TRANSPORT INFORMATION IS IN BYTES 1 & 2. THE DECK WILL NOW RETURN *)
(* STATUS BYTES BEGINNING WITH BYTE 1 (HIGH BITS = 1) WITH A TOTAL   *)
(* OF 2 BYTES (LOW BITS = 2).  THE VARIABLE "REQ_STATUS_MODE" IS     *)
(* USED TO TRACK WHICH STATUS DATA WERE REQUESTED.                   *)

PUSH[PANEL,32]                         (* CHECK IF TAPE IS IN DECK *)
{
  REQ_STATUS_MODE = 2
  CALL 'SEND TO DA-88' (DAT,"$61,$20,$01")
}
(* "CASSETTE OUT" IS BIT 6 OF BYTE 0. THE DECK WILL NOW RETURN       *)
(* STATUS BYTES BEGINNING WITH BYTE 0 (HIGH BITS = 0) WITH A TOTAL   *)
(* OF 1 BYTE (LOW BITS = 1). THE VARIABLE "TAPE_OUT" WILL STORE THIS *)
(* INFORMATION.                                                      *)

IF (LENGTH_STRING(DAT_BUFFER))
{
  IF (((DAT_BUFFER[1] BAND $F0) = $70)
       AND (DAT_BUFFER[2] = $20))                (* STATUS DATA COMMAND *)
  {
    IF (REQ_STATUS_MODE = 1)
    {
      [DAT,241] = DAT_BUFFER[3] BAND $01         (* PLAY = BYTE 1, BIT 1 *)
      [DAT,242] = DAT_BUFFER[3] BAND $20         (* STOP = BYTE 1, BIT 6 *)
      (* ETC. *)
    }
    ELSE IF (REQ_STATUS_MODE = 2)
    {
      TAPE_OUT = DAT_BUFFER[3] BAND $20
    }
    REQ_STATUS_MODE = 0
    DAT_BUFFER = ""
  }
  ELSE IF (FIND_STRING(DAT_BUFFER,"$10,$01",1))  (* ACK *)
    DAT_BUFFER = ""
  ELSE IF (FIND_STRING(DAT_BUFFER,"$11,$12,$01",1))   (* UNDEFINED COMMAND *)
    DAT_BUFFER = ""
}

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)

